#include "game.h"
#include "enemy.h"
#include "randomize.h"

ENEMY::ENEMY()
{
	good = false;
	ex = 0;
	ey = 0;
	move_timer = 0;
	move_speed = 0;
	animate_timer = 0;
	animate_speed = 0;
	body = NULL;
	type = RANDOM;
	direction = NONE;
	steps = 0;
	frame_x = 0;
	frame_y = 0;
	sprite_x = 0;
	sprite_y = 0;
	face = NONE;
	animate_positive = false;
	cText = NULL;
}

ENEMY::~ENEMY()
{
}

void ENEMY::action(int px, int py, bool refresh)
{
	validate_animate();
	switch(type)
	{
	case RANDOM:
		pick_direction();
		move_direction();
		break;
	case HUNT:
		pick_direction(px, py);
		move_direction();
		break;
	case TEMP:
		if(refresh == true)
			type = temp_type;
	case FROZEN:
		//Not complete
		break;
	case BOUNCE_HORIZONTAL:
		if(direction == NONE)
			pick_horizontal();
		move_direction();		
		break;
	case BOUNCE_VERTICAL:
		if(direction == NONE)
			pick_vertical();
		move_direction();		
		break;
	case WALL_LEFT:
		//Not complete
		break;
	case WALL_RIGHT:
		//Not complete
		break;
	case ARCHER:
		//actions not carried out in class (see level.cpp)
		break;
	case RAM:
		if(direction != NONE)
			move_direction();
		break;
	}	
}

void ENEMY::pick_horizontal()
{
	
	switch(direction)
	{
	case NONE:
        direction = EAST;
		break;
	case EAST:
		direction = WEST;
		break;
	case WEST:
		direction = EAST;
		break;
	}
}

void ENEMY::pick_vertical()
{
	
	switch(direction)
	{
	case NONE:
        direction = NORTH;
		break;
	case NORTH:
		direction = SOUTH;
		break;
	case SOUTH:
		direction = NORTH;
		break;
	}
}


void ENEMY::validate_animate()
{
	int height = TILE_H * 2;
	if(type == FROZEN)
		return;
	if(animate_ok())
	{
	switch(direction)
	{
	case NORTHEAST:
	case NORTHWEST:
	case NORTH:
		frame_y = height * 2 + (sprite_y * (6 * height));
		change_animateX();
		break;
	case SOUTHEAST:
	case SOUTHWEST:
	case SOUTH:
		frame_y = height * 0 + (sprite_y * (6 * height));
		change_animateX();
		break;
	case EAST:
		frame_y = height * 3 + (sprite_y * (6 * height));
		change_animateX();
		break;
	case WEST:
		frame_y = height * 1 + (sprite_y * (6 * height));
		change_animateX();
		break;
	case NONE:
		frame_x = (TILE_W * 1) + (sprite_x * (4 * TILE_W));
		break;
	}
	}
}

void ENEMY::change_animateX()
{
	if(animate_positive == true)
		frame_x += TILE_W;
	else
		frame_x -= TILE_W;

	if(frame_x >= (TILE_W * 2) + (sprite_x * (4 * TILE_W)))	animate_positive = false;
	if(frame_x <= (TILE_W * 0) + (sprite_x * (4 * TILE_W)))		animate_positive = true;
}

void ENEMY::pick_direction()
{
	int dir = 0;
	if(steps <= 0)
	{
		dir = rnd(8);
		if(dir == 0)
			direction = NONE;
		else if(dir == 1)
			direction = EAST;			
		else if(dir == 2)
			direction = WEST;
		else if(dir == 3)
			direction = NORTH;
		else if(dir == 4)
			direction = SOUTH;
		else if(dir == 5)
			direction = NORTHWEST;
		else if(dir == 6)
			direction = NORTHEAST;
		else if(dir == 7)
			direction = SOUTHEAST;
		else if(dir == 8)
			direction = SOUTHWEST;
		
		steps = 15;
		face = direction;
	}
	
}

void ENEMY::pick_direction(int x, int y)
{
	if(x < ex && y < ey)
		direction = NORTHWEST;
	if(x < ex && y > ey)
		direction = SOUTHWEST;
	if(x > ex && y < ey)
		direction = NORTHEAST;
	if(x > ex && y > ey)
		direction = SOUTHEAST;
	if(x > ex && y > ey - TILE_H && y < ey + TILE_H)
		direction = EAST;
	if(x < ex && y > ey - TILE_H && y < ey + TILE_H)
		direction = WEST;
	if(y > ey && x > ex - TILE_W && x < ex + TILE_W)
		direction = SOUTH;
	if(y < ey && x > ex - TILE_W && x < ex + TILE_W)
		direction = NORTH;
	steps = 0;
}

void ENEMY::move_direction()
{
	//Move - could be a function
	if(movement_ok())
	{
		switch(direction)
		{
		case NORTH:
			ey -= 5;
			break;
		case SOUTH:
			ey += 5;
			break;
		case WEST:
			ex -= 5;
			break;
		case EAST:
			ex += 5;
			break;
		case NORTHEAST:
			ex += 5;
			ey -= 5;
			break;
		case NORTHWEST:
			ex -= 5;
			ey -= 5;
			break;
		case SOUTHEAST:
			ex += 5;
			ey += 5;
			break;
		case SOUTHWEST:
			ex -= 5;
			ey += 5;
			break;
		}		
		
			steps--;
	}		

}

bool ENEMY::movement_ok()
{
	if(abs(system_timer - move_timer) > move_speed)
	{
		move_timer = system_timer;
		return true;
	}
	return false;
}

bool ENEMY::animate_ok()
{
	if(abs(system_timer - animate_timer) > animate_speed)
	{
		animate_timer = system_timer;
		return true;
	}
	return false;
}



